# standard imports
import unittest
import os
import logging

# external imports
from chainlib.eth.unittest.ethtester import EthTesterCase
from chainlib.eth.nonce import RPCNonceOracle
from chainlib.eth.gas import OverrideGasOracle
from chainlib.connection import RPCConnection
from chainlib.eth.tx import TxFactory
from chainlib.eth.tx import receipt
from hexathon import strip_0x

# local imports
from eth_owned import ERC173

logging.basicConfig(level=logging.DEBUG)
logg = logging.getLogger()

script_dir = os.path.realpath(os.path.dirname(__file__))


class TestOwned(EthTesterCase):

    def setUp(self):
        super(TestOwned, self).setUp()
        self.conn = RPCConnection.connect(self.chain_spec, 'default')
        nonce_oracle = RPCNonceOracle(self.accounts[0], self.conn)

        f = open(os.path.join(script_dir, '..', 'data', 'OwnedSimple.bin'))
        code = f.read()
        f.close()

        txf = TxFactory(self.chain_spec, signer=self.signer, nonce_oracle=nonce_oracle)
        tx = txf.template(self.accounts[0], None, use_nonce=True)
        tx = txf.set_code(tx, code)
        (tx_hash_hex, o) = txf.build(tx)

        r = self.conn.do(o)
        logg.debug('Owned test conrtact published with hash {}'.format(r))

        o = receipt(tx_hash_hex)
        r = self.conn.do(o)
        self.address = r['contract_address']



# standard imports
import unittest
import os
import logging

# external imports
from chainlib.eth.unittest.ethtester import EthTesterCase
from chainlib.eth.nonce import RPCNonceOracle
from chainlib.eth.gas import OverrideGasOracle
from chainlib.connection import RPCConnection
from chainlib.eth.tx import (
        TxFactory,
        receipt,
        )
from hexathon import strip_0x

# local imports
from eth_owned.owned import ERC173

logging.basicConfig(level=logging.DEBUG)
logg = logging.getLogger()

script_dir = os.path.realpath(os.path.dirname(__file__))


class TestInterface: #(EthTesterCase):

    def test_owned(self):
        c = ERC173(self.chain_spec)
        o = c.owner(self.address, sender_address=self.accounts[0])
        r = self.conn.do(o)
        owner = c.parse_owner(r)
        self.assertEqual(owner, strip_0x(self.accounts[0]))


    def test_transfer_ownership(self):
        nonce_oracle = RPCNonceOracle(self.accounts[2], self.conn)
        gas_oracle = OverrideGasOracle(limit=8000000, conn=self.conn)
        c = ERC173(self.chain_spec, nonce_oracle=nonce_oracle, gas_oracle=gas_oracle, signer=self.signer)
        (tx_hash_hex, o) = c.transfer_ownership(self.address, self.accounts[2], self.accounts[1])
        r = self.conn.do(o)

        o = receipt(tx_hash_hex)
        r = self.conn.do(o)
        self.assertEqual(r['status'], 0)
        
        nonce_oracle = RPCNonceOracle(self.accounts[0], self.conn)
        c = ERC173(self.chain_spec, nonce_oracle=nonce_oracle, gas_oracle=gas_oracle, signer=self.signer)
        (tx_hash_hex, o) = c.transfer_ownership(self.address, self.accounts[0], self.accounts[1])
        r = self.conn.do(o)

        o = receipt(tx_hash_hex)
        r = self.conn.do(o)
        self.assertEqual(r['status'], 1)
 
        o = c.owner(self.address, sender_address=self.accounts[0])
        r = self.conn.do(o)
        owner = c.parse_owner(r)
        self.assertEqual(owner, strip_0x(self.accounts[1]))
      

#    def test_accept_ownership(self):
#        nonce_oracle = RPCNonceOracle(self.accounts[0], self.conn)
#        gas_oracle = OverrideGasOracle(limit=8000000, conn=self.conn)
#        c = Owned(self.chain_spec, nonce_oracle=nonce_oracle, gas_oracle=gas_oracle, signer=self.signer)
#        (tx_hash_hex, o) = c.transfer_ownership(self.address, self.accounts[0], self.accounts[1])
#        r = self.conn.do(o)
#
#        nonce_oracle = RPCNonceOracle(self.accounts[2], self.conn)
#        c = Owned(self.chain_spec, nonce_oracle=nonce_oracle, gas_oracle=gas_oracle, signer=self.signer)
#        (tx_hash_hex, o) = c.accept_ownership(self.address, self.accounts[2])
#        r = self.conn.do(o)
#
#        o = receipt(tx_hash_hex)
#        r = self.conn.do(o)
#        self.assertEqual(r['status'], 0)
#
#        nonce_oracle = RPCNonceOracle(self.accounts[1], self.conn)
#        c = Owned(self.chain_spec, nonce_oracle=nonce_oracle, gas_oracle=gas_oracle, signer=self.signer)
#        (tx_hash_hex, o) = c.accept_ownership(self.address, self.accounts[1])
#        r = self.conn.do(o)
#
#        o = receipt(tx_hash_hex)
#        r = self.conn.do(o)
#        self.assertEqual(r['status'], 1)
#
#        o = c.owner(self.address, sender_address=self.accounts[0])
#        r = self.conn.do(o)
#        owner = c.parse_owner(r)
#        self.assertEqual(owner, strip_0x(self.accounts[1]))
#
#
#    def test_take_ownership(self):
#        nonce_oracle = RPCNonceOracle(self.accounts[0], self.conn)
#        gas_oracle = OverrideGasOracle(limit=8000000, conn=self.conn)
#        c = Owned(self.chain_spec, nonce_oracle=nonce_oracle, gas_oracle=gas_oracle, signer=self.signer)
#        (tx_hash_hex, o) = c.transfer_ownership(self.address, self.accounts[0], self.address)
#        r = self.conn.do(o)
#
#        (tx_hash_hex, o) = c.take_ownership(self.address, self.accounts[0], self.address)
#        r = self.conn.do(o)
#
#        o = receipt(tx_hash_hex)
#        r = self.conn.do(o)
#        self.assertEqual(r['status'], 1)
#
#        o = c.owner(self.address, sender_address=self.accounts[0])
#        r = self.conn.do(o)
#        owner = c.parse_owner(r)
#        self.assertEqual(owner, strip_0x(self.address))
#
#
#    def test_ownership_final(self):
#        nonce_oracle = RPCNonceOracle(self.accounts[0], self.conn)
#        gas_oracle = OverrideGasOracle(limit=8000000, conn=self.conn)
#        c = Owned(self.chain_spec, nonce_oracle=nonce_oracle, gas_oracle=gas_oracle, signer=self.signer)
#        (tx_hash_hex, o) = c.transfer_ownership(self.address, self.accounts[0], self.accounts[1], final=True)
#        r = self.conn.do(o)
#
#        c = Owned(self.chain_spec, nonce_oracle=nonce_oracle, gas_oracle=gas_oracle, signer=self.signer)
#        (tx_hash_hex, o) = c.transfer_ownership(self.address, self.accounts[0], self.accounts[1], final=True)
#        r = self.conn.do(o)
#        o = receipt(tx_hash_hex)
#        r = self.conn.do(o)
#        self.assertEqual(r['status'], 0)

if __name__ == '__main__':
    unittest.main()



"""Query owner (EIP 173) of contract

.. moduleauthor:: Louis Holbrook <dev@holbrook.no>
.. pgp:: 0826EDA1702D1E87C6E2875121D2E7BB88C2A746 

"""

# standard imports
import sys
import os
import json
import argparse
import logging

# external imports
from funga.eth.signer import EIP155Signer
from funga.eth.keystore.dict import DictKeystore
from chainlib.chain import ChainSpec
from chainlib.eth.connection import EthHTTPConnection
from chainlib.error import JSONRPCException

# local imports
from eth_owned.owned import Owned

logging.basicConfig(level=logging.WARNING)
logg = logging.getLogger()

script_dir = os.path.dirname(__file__)
data_dir = os.path.join(script_dir, '..', 'data')

default_format = 'terminal'

argparser = argparse.ArgumentParser()
argparser.add_argument('-p', '--provider', dest='p', default='http://localhost:8545', type=str, help='RPC provider url (http only)')
argparser.add_argument('-i', '--chain-spec', dest='i', type=str, default='evm:ethereum:1', help='Chain specification string')
#argparser.add_argument('-a', '--contract-address', dest='a', required=True, type=str, help='Contract address')
#argparser.add_argument('-f', '--format', dest='f', type=str, default=default_format, help='Output format [human, brief]')
argparser.add_argument('-v', action='store_true', help='Be verbose')
argparser.add_argument('-vv', action='store_true', help='Be more verbose')
argparser.add_argument('--env-prefix', default=os.environ.get('CONFINI_ENV_PREFIX'), dest='env_prefix', type=str, help='environment prefix for variables to overwrite configuration')
argparser.add_argument('address', type=str, nargs='?', help='Address to check registration for')
args = argparser.parse_args()

if args.vv:
    logg.setLevel(logging.DEBUG)
elif args.v:
    logg.setLevel(logging.INFO)

chain_spec = ChainSpec.from_chain_str(args.i)

rpc = EthHTTPConnection(args.p)
#account_registry_address = args.a
address = args.address


def main():
    c = Owned(chain_spec)
    o = c.owner(address)

    r = rpc.do(o)
    owner = c.parse_owner(r)
    print(owner)


if __name__ == '__main__':
    main()



"""Deploys the void owner contract

.. moduleauthor:: Louis Holbrook <dev@holbrook.no>
.. pgp:: 0826EDA1702D1E87C6E2875121D2E7BB88C2A746 

"""

# standard imports
import sys
import os
import json
import argparse
import logging

# external imports
from funga.eth.signer import EIP155Signer
from funga.eth.keystore.dict import DictKeystore
from chainlib.chain import ChainSpec
from chainlib.eth.nonce import (
        RPCNonceOracle,
        OverrideNonceOracle,
        )
from chainlib.eth.gas import (
        RPCGasOracle,
        OverrideGasOracle,
        )
from chainlib.eth.connection import EthHTTPConnection
from chainlib.eth.tx import receipt

# local imports
from eth_owned.void import VoidOwner

logging.basicConfig(level=logging.WARNING)
logg = logging.getLogger()

script_dir = os.path.dirname(__file__)
data_dir = os.path.join(script_dir, '..', 'data')

argparser = argparse.ArgumentParser()
argparser.add_argument('-p', '--provider', dest='p', default='http://localhost:8545', type=str, help='Web3 provider url (http only)')
argparser.add_argument('-w', action='store_true', help='Wait for the last transaction to be confirmed')
argparser.add_argument('-ww', action='store_true', help='Wait for every transaction to be confirmed')
argparser.add_argument('-i', '--chain-spec', dest='i', type=str, default='evm:ethereum:1', help='Chain specification string')
argparser.add_argument('-y', '--key-file', dest='y', type=str, help='Ethereum keystore file to use for signing')
argparser.add_argument('-v', action='store_true', help='Be verbose')
argparser.add_argument('-vv', action='store_true', help='Be more verbose')
argparser.add_argument('-d', action='store_true', help='Dump RPC calls to terminal and do not send')
argparser.add_argument('--gas-price', type=int, dest='gas_price', help='Override gas price')
argparser.add_argument('--nonce', type=int, help='Override transaction nonce')
argparser.add_argument('--env-prefix', default=os.environ.get('CONFINI_ENV_PREFIX'), dest='env_prefix', type=str, help='environment prefix for variables to overwrite configuration')
args = argparser.parse_args()

if args.vv:
    logg.setLevel(logging.DEBUG)
elif args.v:
    logg.setLevel(logging.INFO)

block_last = args.w
block_all = args.ww

passphrase_env = 'ETH_PASSPHRASE'
if args.env_prefix != None:
    passphrase_env = args.env_prefix + '_' + passphrase_env
passphrase = os.environ.get(passphrase_env)
if passphrase == None:
    logg.warning('no passphrase given')
    passphrase=''

signer_address = None
keystore = DictKeystore()
if args.y != None:
    logg.debug('loading keystore file {}'.format(args.y))
    signer_address = keystore.import_keystore_file(args.y, password=passphrase)
    logg.debug('now have key for signer address {}'.format(signer_address))
signer = EIP155Signer(keystore)

chain_spec = ChainSpec.from_chain_str(args.i)

rpc = EthHTTPConnection(args.p)
nonce_oracle = None
if args.nonce != None:
    nonce_oracle = OverrideNonceOracle(signer_address, args.nonce)
else:
    nonce_oracle = RPCNonceOracle(signer_address, rpc)

gas_oracle = None
if args.gas_price !=None:
    gas_oracle = OverrideGasOracle(price=args.gas_price, conn=rpc, code_callback=VoidOwner.gas)
else:
    gas_oracle = RPCGasOracle(rpc, code_callback=VoidOwner.gas)

dummy = args.d


def main():
    c = VoidOwner(chain_spec, signer=signer, gas_oracle=gas_oracle, nonce_oracle=nonce_oracle)
    (tx_hash_hex, o) = c.constructor(signer_address)
    if dummy:
        print(tx_hash_hex)
        print(o)
    else:
        rpc.do(o)
        if block_last:
            r = rpc.wait(tx_hash_hex)
            if r['status'] == 0:
                sys.stderr.write('EVM revert while deploying contract. Wish I had more to tell you')
                sys.exit(1)
            # TODO: pass through translator for keys (evm tester uses underscore instead of camelcase)
            address = r['contractAddress']

            print(address)
        else:
            print(tx_hash_hex)


if __name__ == '__main__':
    main()



# external imports
from hexathon import (
        add_0x,
    )

# local imports
from chainlib.eth.contract import (
        ABIContractEncoder,
        ABIContractDecoder,
        ABIContractType,
        abi_decode_single,
    )
from chainlib.jsonrpc import JSONRPCRequest
from chainlib.eth.tx import (
        TxFactory,
        TxFormat,
        )
from chainlib.eth.constant import ZERO_ADDRESS

class ERC173(TxFactory):

    def transfer_ownership(self, contract_address, sender_address, new_owner_address, final=False, tx_format=TxFormat.JSONRPC):
        enc = ABIContractEncoder()
        if final:
            enc.method('transferOwnershipFinal')
        else:
            enc.method('transferOwnership')
        enc.typ(ABIContractType.ADDRESS)
        enc.address(new_owner_address)
        data = add_0x(enc.get())
        tx = self.template(sender_address, contract_address, use_nonce=True)
        tx = self.set_code(tx, data)
        tx = self.finalize(tx, tx_format)
        return tx


    def owner(self, contract_address, sender_address=ZERO_ADDRESS, id_generator=None):
        j = JSONRPCRequest(id_generator)
        o = j.template()
        o['method'] = 'eth_call'
        enc = ABIContractEncoder()
        enc.method('owner')
        data = add_0x(enc.get())
        tx = self.template(sender_address, contract_address)
        tx = self.set_code(tx, data)
        o['params'].append(self.normalize(tx))
        o['params'].append('latest')
        o = j.finalize(o)
        return o


    @classmethod
    def parse_owner(self, v):
        return abi_decode_single(ABIContractType.ADDRESS, v)



# standard imports
import logging
import json
import os

# external imports
from chainlib.eth.tx import (
        TxFactory,
        TxFormat,
        )
from chainlib.eth.contract import (
        ABIContractEncoder,
        ABIContractDecoder,
        ABIContractType,
        abi_decode_single,
        )
from chainlib.eth.constant import ZERO_ADDRESS
from chainlib.eth.error import RequestMismatchException
from hexathon import (
        add_0x,
        strip_0x,
        )

logg = logging.getLogger()

moddir = os.path.dirname(__file__)
datadir = os.path.join(moddir, 'data')


class VoidOwner(TxFactory):

    __abi = None
    __bytecode = None

    @staticmethod
    def abi():
        if VoidOwner.__abi == None:
            f = open(os.path.join(datadir, 'VoidOwner.json'), 'r')
            VoidOwner.__abi = json.load(f)
            f.close()
        return VoidOwner.__abi


    @staticmethod
    def bytecode():
        if VoidOwner.__bytecode == None:
            f = open(os.path.join(datadir, 'VoidOwner.bin'))
            VoidOwner.__bytecode = f.read()
            f.close()
        return VoidOwner.__bytecode


    @staticmethod
    def gas(code=None):
        return 500000


    def constructor(self, sender_address):
        code = VoidOwner.bytecode()
        tx = self.template(sender_address, None, use_nonce=True)
        tx = self.set_code(tx, code)
        return self.build(tx)



# standard imports
import unittest

# local imports
from eth_owned.unittest import TestOwned as TestOwnedBase
from eth_owned.unittest import TestInterface


class TestOwned(TestOwnedBase, TestInterface):
    pass


if __name__ == '__main__':
    unittest.main()



from setuptools import setup

requirements = []
f = open('requirements.txt', 'r')
while True:
    l = f.readline()
    if l == '':
        break
    requirements.append(l.rstrip())
f.close()

test_requirements = []
f = open('test_requirements.txt', 'r')
while True:
    l = f.readline()
    if l == '':
        break
    test_requirements.append(l.rstrip())
f.close()


setup(
        package_data={
            '': [
                'data/*.abi.json',
                'data/*.bin',
                ],
            },
        include_package_data=True,
        install_requires=requirements,
        tests_require=test_requirements,
        )
